@@ -4,9 +4,14 @@
 import logging
 from aioboto3.session import Session
 
-__author__ = """Terry Cain"""
-__email__ = 'terry@terrys-home.co.uk'
-__version__ = '13.1.0'
+__author__ = """Terri Cain"""
+__email__ = 'terri@dolphincorp.co.uk'
+
+
+try:
+    from aioboto3._version import __version__
+except PackageNotFoundError:
+    __version__ = "0.0.0"
 
 
 # Set up logging to ``/dev/null`` like a library is supposed to.
