@@ -104,11 +104,11 @@ async def download_file(
         )
 
 
-async def _download_part(self, bucket: str, key: str, headers: Dict[str, str], start: int, file: AnyFileObject, semaphore: asyncio.Semaphore, write_lock: asyncio.Lock,
+async def _download_part(self, bucket: str, key: str, extraArgs: Dict[str, str], headers: Dict[str, str], start: int, file: AnyFileObject, semaphore: asyncio.Semaphore, write_lock: asyncio.Lock,
                          callback=None, io_queue: Optional[asyncio.Queue] = None) -> None:
     async with semaphore:  # limit number of concurrent downloads
         response = await self.get_object(
-            Bucket=bucket, Key=key, Range=headers['Range']
+            Bucket=bucket, Key=key, Range=headers['Range'], **extraArgs
         )
         content = await response['Body'].read()
 
@@ -274,7 +274,7 @@ async def download_fileobj(
             headers = {'Range': f'bytes={start}-{end - 1}'}
             # Create a task for each part download
             tasks.append(
-                _download_part(self, Bucket, Key, headers, start, Fileobj, semaphore, write_mutex, wrapper_callback, io_queue if not is_seekable else None)
+                _download_part(self, Bucket, Key, ExtraArgs, headers, start, Fileobj, semaphore, write_mutex, wrapper_callback, io_queue if not is_seekable else None)
             )
 
         # Run all the download tasks concurrently
@@ -283,7 +283,7 @@ async def download_fileobj(
         if queue_reader_future:
             await queue_reader_future
 
-        logger.info(f'Downloaded file from {Bucket}/{Key}')
+        logger.debug(f'Downloaded file from {Bucket}/{Key}')
 
     except ClientError as e:
         raise Exception(
@@ -297,7 +297,7 @@ async def upload_fileobj(
     Bucket: str,
     Key: str,
     ExtraArgs: Optional[Dict[str, Any]] = None,
-    Callback: Optional[Callable[[int], None]] = None,
+    Callback: Optional[TransferCallback] = None,
     Config: Optional[S3TransferConfig] = None,
     Processing: Callable[[bytes], bytes] = None
 ):
@@ -378,7 +378,10 @@ async def upload_fileobj(
             **kwargs
         )
         if Callback:
-            Callback(len(initial_data))
+            if inspect.iscoroutinefunction(Callback):
+                await Callback(len(initial_data))
+            else:
+                Callback(len(initial_data))
         return
 
     # File bigger than threshold, start multipart upload
@@ -423,7 +426,10 @@ async def upload_fileobj(
             # Call the callback, if it blocks then not good :/
             if Callback:
                 try:
-                    Callback(current_bytes)
+                    if inspect.iscoroutinefunction(Callback):
+                        await Callback(current_bytes)
+                    else:
+                        Callback(current_bytes)
                 except:  # noqa: E722
                     pass
 
@@ -621,10 +627,14 @@ async def copy(
     finished_parts = []
     total_size = 0
 
+    sem = asyncio.Semaphore(Config.max_request_concurrency)
+
     async def uploader(size: int, part_args: Dict[str, Any]):
         nonlocal total_size
 
-        upload_part_response = await self.upload_part_copy(**part_args)
+        async with sem:
+            upload_part_response = await self.upload_part_copy(**part_args)
+
         finished_parts.append({'ETag': upload_part_response['CopyPartResult']['ETag'], 'PartNumber': part_args['PartNumber']})
 
         # Call the callback, if it blocks then not good :/
