@@ -6,15 +6,14 @@ an async botocore session
 
 import copy
 
-import aiobotocore.session
-from aiobotocore.config import AioConfig
-
 import boto3.session
 import boto3.resources.base
 import boto3.utils
-from botocore.exceptions import DataNotFoundError, UnknownServiceError
+from boto3.session import DataNotFoundError, UnknownServiceError
 from boto3.exceptions import ResourceNotExistsError, UnknownAPIVersionError
 
+import aiobotocore.session
+from aiobotocore.config import AioConfig
 from aioboto3.resources.factory import AIOBoto3ResourceFactory
 
 
@@ -31,8 +30,8 @@ class Session(boto3.session.Session):
     :param aws_session_token: AWS temporary session token
     :type region_name: string
     :param region_name: Default region when creating new connections
-    :type botocore_session: botocore.session.Session
-    :param botocore_session: Use this Botocore session instead of creating
+    :type botocore_session: aiobotocore.session.AioSession
+    :param botocore_session: Use this AioBotocore session instead of creating
                              a new default one.
     :type profile_name: string
     :param profile_name: The name of a profile to use. If not given, then
