@@ -4,13 +4,14 @@ import tempfile
 from io import BytesIO
 
 from botocore.exceptions import ClientError
+from boto3.s3.transfer import S3TransferConfig
 import aiofiles
 import pytest
 
 
 @pytest.mark.asyncio
-async def test_s3_download_file(event_loop, s3_client, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_download_file(event_loop, s3_client, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
     await s3_client.put_object(Bucket=bucket_name, Key='test_file', Body=b'Hello World\n')
 
     download_file = '/tmp/aioboto3_temp_s3_download.txt'
@@ -38,9 +39,9 @@ async def test_s3_download_file(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_download_fileobj(event_loop, s3_client, bucket_name):
+async def test_s3_download_fileobj(event_loop, s3_client, bucket_name, region):
     data = b'Hello World\n'
-    await s3_client.create_bucket(Bucket=bucket_name)
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
     await s3_client.put_object(Bucket=bucket_name, Key='test_file', Body=data)
 
     fh = BytesIO()
@@ -51,8 +52,48 @@ async def test_s3_download_fileobj(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_download_file_404(event_loop, s3_client, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_download_fileobj_nonseekable_asyncwrite(event_loop, s3_client, bucket_name, region):
+    data = b'Hello World\n'
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
+    await s3_client.put_object(Bucket=bucket_name, Key='test_file', Body=data)
+
+    class FileObj:
+        def __init__(self) -> None:
+            self.data = b''
+
+        async def write(self, b: bytes) -> int:
+            self.data += b
+            return len(b)
+
+    fh = FileObj()
+    await s3_client.download_fileobj(bucket_name, 'test_file', fh)
+
+    assert fh.data == data
+
+
+@pytest.mark.asyncio
+async def test_s3_download_fileobj_nonseekable_syncwrite(event_loop, s3_client, bucket_name, region):
+    data = b'Hello World\n'
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
+    await s3_client.put_object(Bucket=bucket_name, Key='test_file', Body=data)
+
+    class FileObj:
+        def __init__(self) -> None:
+            self.data = b''
+
+        def write(self, b: bytes) -> int:
+            self.data += b
+            return len(b)
+
+    fh = FileObj()
+    await s3_client.download_fileobj(bucket_name, 'test_file', fh)
+
+    assert fh.data == data
+
+
+@pytest.mark.asyncio
+async def test_s3_download_file_404(event_loop, s3_client, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     try:
         await s3_client.download_file(bucket_name, 'test_file', '/tmp/somefile')
@@ -62,9 +103,9 @@ async def test_s3_download_file_404(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_upload_fileobj(event_loop, s3_client, bucket_name):
+async def test_s3_upload_fileobj(event_loop, s3_client, bucket_name, region):
     data = b'Hello World\n'
-    await s3_client.create_bucket(Bucket=bucket_name)
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     fh = BytesIO()
     fh.write(data)
@@ -86,8 +127,8 @@ async def test_s3_upload_fileobj(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_upload_empty_fileobj(event_loop, s3_client, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_upload_empty_fileobj(event_loop, s3_client, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     fh = BytesIO(b'')
     fh.seek(0)
@@ -108,8 +149,8 @@ async def test_s3_upload_empty_fileobj(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_upload_fileobj_async(event_loop, s3_client, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_upload_fileobj_async(event_loop, s3_client, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     data = b'Hello World\n'
 
@@ -126,7 +167,47 @@ async def test_s3_upload_fileobj_async(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_upload_broken_fileobj(event_loop, s3_client, bucket_name):
+async def test_s3_upload_fileobj_async_multipart(event_loop, s3_client, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
+
+    data = b'Hello World\n'
+
+    tmpfile = tempfile.NamedTemporaryFile(delete=False)
+    tmpfile.close()
+    async with aiofiles.open(tmpfile.name, mode='wb') as fpw:
+        await fpw.write(data)
+
+    async with aiofiles.open(tmpfile.name, mode='rb') as fpr:
+        config = S3TransferConfig(multipart_threshold=4)
+        await s3_client.upload_fileobj(fpr, bucket_name, 'test_file', Config=config)
+
+    resp = await s3_client.get_object(Bucket=bucket_name, Key='test_file')
+    assert (await resp['Body'].read()) == data
+
+
+@pytest.mark.asyncio
+async def test_s3_upload_fileobj_async_slow(event_loop, s3_client, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
+
+    data = b'Hello World\n'
+
+    class FakeFile:
+        def __init__(self, filedata: bytes) -> None:
+            self._data = filedata
+
+        async def read(self, numbytes: int) -> bytes:
+            result = self._data[:5]
+            self._data = self._data[5:]
+            return result
+
+    await s3_client.upload_fileobj(FakeFile(data), bucket_name, 'test_file')
+
+    resp = await s3_client.get_object(Bucket=bucket_name, Key='test_file')
+    assert (await resp['Body'].read()) == data
+
+
+@pytest.mark.asyncio
+async def test_s3_upload_broken_fileobj(event_loop, s3_client, bucket_name, region):
     class BrokenFile(object):
         def __init__(self, data: bytes):
             self._data = data
@@ -134,7 +215,7 @@ async def test_s3_upload_broken_fileobj(event_loop, s3_client, bucket_name):
         def read(self, count):
             raise IOError("some bad file")
 
-    await s3_client.create_bucket(Bucket=bucket_name)
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     fh = BrokenFile(b'Hello World\n')
     try:
@@ -147,9 +228,9 @@ async def test_s3_upload_broken_fileobj(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_upload_fileobj_with_transform(event_loop, s3_client, bucket_name):
+async def test_s3_upload_fileobj_with_transform(event_loop, s3_client, bucket_name, region):
     data = b'Hello World\n'
-    await s3_client.create_bucket(Bucket=bucket_name)
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     fh = BytesIO()
     fh.write(data)
@@ -164,10 +245,10 @@ async def test_s3_upload_fileobj_with_transform(event_loop, s3_client, bucket_na
 
 
 @pytest.mark.asyncio
-async def test_s3_upload_file(event_loop, s3_client, bucket_name):
+async def test_s3_upload_file(event_loop, s3_client, bucket_name, region):
     data = b'Hello World\n'
     filename = '/tmp/aioboto3_temp_s3_upload.txt'
-    await s3_client.create_bucket(Bucket=bucket_name)
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     open(filename, 'wb').write(data)
 
@@ -178,13 +259,11 @@ async def test_s3_upload_file(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_copy(event_loop, s3_client, bucket_name):
+async def test_s3_copy(event_loop, s3_client, bucket_name, region):
     data = b'Hello World\n'
 
-    # TODO ============================================= generate largeish file
-
     filename = '/tmp/aioboto3_temp_s3_upload.txt'
-    await s3_client.create_bucket(Bucket=bucket_name)
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     # Upload file
     open(filename, 'wb').write(data)
@@ -200,8 +279,49 @@ async def test_s3_copy(event_loop, s3_client, bucket_name):
 
 
 @pytest.mark.asyncio
-async def test_s3_resource_objects_all(event_loop, s3_client, s3_resource, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_copy_multipart(event_loop, s3_client, bucket_name, region):
+    data = b'Hello World\n'
+
+    filename = '/tmp/aioboto3_temp_s3_upload.txt'
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
+
+    # Upload file
+    open(filename, 'wb').write(data)
+    await s3_client.upload_file(filename, bucket_name, 'test_file')
+
+    # Copy file
+    copy_source = {'Bucket': bucket_name, 'Key': 'test_file'}
+    config = S3TransferConfig(multipart_threshold=4)
+    await s3_client.copy(copy_source, bucket_name, 'test_file2', Config=config, ExtraArgs={'RequestPayer': 'requester'})
+
+    # Get copied file
+    resp = await s3_client.get_object(Bucket=bucket_name, Key='test_file2')
+    assert (await resp['Body'].read()) == data
+
+
+@pytest.mark.asyncio
+async def test_s3_copy_from(event_loop, s3_client, s3_resource, bucket_name, region):
+    data = b'Hello World\n'
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
+
+    fh = BytesIO()
+    fh.write(data)
+    fh.seek(0)
+
+    await s3_client.upload_fileobj(fh, bucket_name, 'test_file')
+
+    resource = await s3_resource.Object(bucket_name, "new_test_file")
+    copy_source = bucket_name + "/test_file"
+    copy_result = await resource.copy_from(CopySource=copy_source)
+    assert 'CopyObjectResult' in copy_result
+
+    resp = await s3_client.get_object(Bucket=bucket_name, Key='new_test_file')
+    assert (await resp['Body'].read()) == data
+
+
+@pytest.mark.asyncio
+async def test_s3_resource_objects_all(event_loop, s3_client, s3_resource, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
     files_to_create = {'test/file1', 'test2/file1', 'test2/file2'}
     for file in files_to_create:
         await s3_client.put_object(Bucket=bucket_name, Key=file, Body=b'Hello World\n')
@@ -216,8 +336,8 @@ async def test_s3_resource_objects_all(event_loop, s3_client, s3_resource, bucke
 
 
 @pytest.mark.asyncio
-async def test_s3_resource_objects_filter(event_loop, s3_client, s3_resource, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_resource_objects_filter(event_loop, s3_client, s3_resource, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
     files_to_create = {'test/file1', 'test2/file1', 'test2/file2'}
     for file in files_to_create:
         await s3_client.put_object(Bucket=bucket_name, Key=file, Body=b'Hello World\n')
@@ -232,8 +352,8 @@ async def test_s3_resource_objects_filter(event_loop, s3_client, s3_resource, bu
 
 
 @pytest.mark.asyncio
-async def test_s3_resource_objects_delete(event_loop, s3_client, s3_resource, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_resource_objects_delete(event_loop, s3_client, s3_resource, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
     files_to_create = {'test/file1', 'test2/file1', 'test2/file2'}
     for file in files_to_create:
         await s3_client.put_object(Bucket=bucket_name, Key=file, Body=b'Hello World\n')
@@ -249,8 +369,8 @@ async def test_s3_resource_objects_delete(event_loop, s3_client, s3_resource, bu
 
 
 @pytest.mark.asyncio
-async def test_s3_resource_objects_delete_filter(event_loop, s3_client, s3_resource, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_resource_objects_delete_filter(event_loop, s3_client, s3_resource, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
     files_to_create = {'test/file1', 'test2/file1', 'test2/file2'}
     for file in files_to_create:
         await s3_client.put_object(Bucket=bucket_name, Key=file, Body=b'Hello World\n')
@@ -267,9 +387,9 @@ async def test_s3_resource_objects_delete_filter(event_loop, s3_client, s3_resou
 
 
 @pytest.mark.asyncio
-async def test_s3_object_summary_load(event_loop, s3_client, s3_resource, bucket_name):
+async def test_s3_object_summary_load(event_loop, s3_client, s3_resource, bucket_name, region):
     data = b'Hello World\n'
-    await s3_client.create_bucket(Bucket=bucket_name)
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
     await s3_client.put_object(Bucket=bucket_name, Key='test_file', Body=data)
 
     obj = await s3_resource.ObjectSummary(bucket_name, 'test_file')
@@ -278,8 +398,8 @@ async def test_s3_object_summary_load(event_loop, s3_client, s3_resource, bucket
 
 
 @pytest.mark.asyncio
-async def test_s3_bucket_creation_date(event_loop, s3_client, s3_resource, bucket_name):
-    await s3_client.create_bucket(Bucket=bucket_name)
+async def test_s3_bucket_creation_date(event_loop, s3_client, s3_resource, bucket_name, region):
+    await s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={'LocationConstraint': region})
 
     bucket = await s3_resource.Bucket(bucket_name)
     creation_date = await bucket.creation_date
